@@ -158,7 +158,7 @@ def log(func=None,
             logger.critical('Failed to log a message: %s', exc, exc_info=True)
 
     class __logged__(Aspect):
-        __slots__ = 'cutpoint_function', 'final_function', 'binding', '__name__'
+        __slots__ = 'cutpoint_function', 'final_function', 'binding', '__name__', '__weakref__'
 
         bind = False
 
