@@ -1,11 +1,15 @@
-from sys import _getframe
 from collections import defaultdict
 from collections import namedtuple
 from difflib import unified_diff
 from functools import partial
 from functools import wraps
 from inspect import isclass
+try:
+    from logging import _levelNames as nameToLevel
+except ImportError:
+    from logging import _nameToLevel as nameToLevel
 from logging import getLogger
+from sys import _getframe
 from traceback import format_stack
 
 from aspectlib import ALL_METHODS
@@ -65,7 +69,82 @@ def mock(return_value, call=False):
     return mock_decorator
 
 
-class RecordingFunctionWrapper(object):
+class LogCapture(object):
+    """
+    Records all log messages made on the given logger. Assumes the logger has a ``_log`` method.
+
+    Example::
+
+        >>> import logging
+        >>> logger = logging.getLogger('mylogger')
+        >>> with LogCapture(logger, level='INFO') as logs:
+        ...     logger.debug("Message from debug: %s", 'somearg')
+        ...     logger.info("Message from info: %s", 'somearg')
+        ...     logger.error("Message from error: %s", 'somearg')
+        >>> logs.calls
+        [('Message from info: %s', ('somearg',), 20), ('Message from error: %s', ('somearg',), 40)]
+        >>> logs.has('Message from info: %s')
+        True
+        >>> logs.has('Message from info: somearg')
+        True
+        >>> logs.has('Message from info: %s', 'badarg')
+        False
+        >>> logs.has('Message from debug: %s')
+        False
+        >>> logs.assertLogged('Message from error: %s')
+        >>> logs.assertLogged('Message from error: %s')
+        >>> logs.assertLogged('Message from error: %s')
+
+
+    """
+    def __init__(self, logger, level='DEBUG'):
+        self._logger = logger
+        self._level = nameToLevel[level]
+        self._calls = []
+        self._rollback = None
+
+    def __enter__(self):
+        self._rollback = weave(
+            self._logger,
+            record(callback=self._callback, extended=True, iscalled=True),
+            methods='_log$'
+        )
+        return self
+
+    def __exit__(self, *exc):
+        self._rollback()
+
+    def _callback(self, _binding, _qualname, args, _kwargs):
+        level, message, args = args
+        if level >= self._level:
+            self._calls.append((message % args if args else message, message, args, level))
+
+    @property
+    def calls(self):
+        return [i[1:] for i in self._calls]
+
+    def has(self, message, *args, **kwargs):
+        level = kwargs.pop('level', None)
+        assert not kwargs, "Unexpected arguments: %s" % kwargs
+        for call_final_message, call_message, call_args, call_level in self._calls:
+            if (
+                (level is None or level == call_level) and (
+                    message == call_message and args == call_args
+                    if args else
+                    message == call_final_message or message == call_message
+                )
+            ):
+                return True
+        return False
+
+    def assertLogged(self, message, *args, **kwargs):
+        if not self.has(message, *args, **kwargs):
+            raise AssertionError("There's no such message %r (with args %r) logged on %s. Logged messages where: %s" % (
+                message, args, self._logger, self.calls
+            ))
+
+
+class _RecordingFunctionWrapper(object):
     """
     Function wrapper that records calls and can be used as an weaver context manager.
 
@@ -123,7 +202,7 @@ class RecordingFunctionWrapper(object):
                 ))
 
     def __get__(self, instance, owner):
-        return RecordingFunctionWrapper(
+        return _RecordingFunctionWrapper(
             self.__wrapped.__get__(instance, owner),
             iscalled=self.__iscalled,
             calls=self.calls,
@@ -193,7 +272,7 @@ def record(func=None, recurse_lock_factory=allocate_lock, **options):
         Added `extended` option.
     """
     if func:
-        return RecordingFunctionWrapper(
+        return _RecordingFunctionWrapper(
             func,
             recurse_lock=recurse_lock_factory(),
             **options
@@ -233,7 +312,7 @@ class StoryResultWrapper(object):
         exec("%s = __unsupported__" % mm)
 
 
-class StoryFunctionWrapper(object):
+class _StoryFunctionWrapper(object):
     def __init__(self, wrapped, handle, binding=None, owner=None):
         self._wrapped = wrapped
         self._name = wrapped.__name__
@@ -263,7 +342,7 @@ class StoryFunctionWrapper(object):
         ), self)
 
 
-class ReplayFunctionWrapper(StoryFunctionWrapper):
+class _ReplayFunctionWrapper(_StoryFunctionWrapper):
     def __call__(self, *args, **kwargs):
         if self._binding is None:
             return self._handle(None, self._qualname, args, kwargs, self._wrapped)
@@ -321,7 +400,7 @@ class _RecordingBase(object):
         self._options.setdefault('methods', ALL_METHODS)
         self.__entanglement = weave(
             self._target,
-            partial(self.FunctionWrapper, handle=self._handle),
+            partial(self._FunctionWrapper, handle=self._handle),
             **self._options
         )
         return self
@@ -380,7 +459,7 @@ class Story(_RecordingBase):
 
         .. [1] http://www.martinfowler.com/bliki/TestDouble.html
     """
-    FunctionWrapper = StoryFunctionWrapper
+    _FunctionWrapper = _StoryFunctionWrapper
 
     def __init__(self, *args, **kwargs):
         super(Story, self).__init__(*args, **kwargs)
@@ -446,7 +525,7 @@ class Replay(_RecordingBase):
     This object should be created by :obj:`Story <aspectlib.test.Story>`'s :obj:`replay <aspectlib.test.Story.replay>`
     method.
     """
-    FunctionWrapper = ReplayFunctionWrapper
+    _FunctionWrapper = _ReplayFunctionWrapper
 
     def __init__(self, play, proxy=True, strict=True, dump=True, recurse_lock=False, **options):
         super(Replay, self).__init__(play._target, **options)
@@ -570,17 +649,17 @@ class Replay(_RecordingBase):
     def expected(self):
         return ''.join(_format_calls(self._expected))
 
-    def __exit__(self, *args):
+    def __exit__(self, *exception):
         super(Replay, self).__exit__()
         if self._strict or self._dump:
             diff = self.diff
             if diff:
-                if self._dump:
+                if exception or self._dump:
                     print('STORY/REPLAY DIFF:')
                     print('    ' + '\n    '.join(diff.splitlines()))
                     print('ACTUAL:')
                     print('    ' + '    '.join(_format_calls(self._actual)))
-                if self._strict:
+                if not exception and self._strict:
                     raise AssertionError(diff)
 
 
